//########################################
//#            Timo Ottersberg           #
//#              April 2004              #
//# Beschleunigungsmesser mit ADX202JE,  #
//#            AT90S2313 und             #
//#       2X16 Zeichen LC-Display        #
//########################################


#include <inttypes.h>
#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>

#define CLK 8000000
#define delay_ms(ms) _delay_loop_2(ms * (CLK/4000))
#define delay_us(us) _delay_loop_2(us * (CLK/4000000))

#define RS 0
#define RD 1
#define EN1 2
#define EN2 3

void lcd_enable(char disp);
void lcd_command(char command, char disp);
void lcd_data(char data, char disp);
void lcd_gotoxy(char x, char y, char disp);
void lcd_init(char disp);
void lcd_print(char *text, char disp);

void show_g(void);

char pos[2][16]={
        {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
        {0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F}
};

char text[6];
volatile int count=0;
int temp=0;
volatile int t2;
volatile int t1;
int ausgabe;
float erg;
int max, min = 0;

SIGNAL(SIG_OUTPUT_COMPARE1A){

  count++;

}

SIGNAL(SIG_INTERRUPT1){

  if(PORTD==0x7F){
    PORTD=0x00;				//PORTD.6 -> Low
    MCUCR = (1<<ISC11 | 1<<ISC10);	//Int1 rising edge
    t1 = count;
  }else{
    PORTD=0x7F;				//PORTD.6 -> high
    MCUCR = (1<<ISC11 | 0<<ISC10);	//Int1 falling edge
    t2=count;
    count = 0;
  }

}

void show_g(void){

  //A(g) = (T1/T2  0.5)/12.5%
  (int) ausgabe = (((((float)t1/(float)t2)*1000)-461)*1000)/125;

  itoa(ausgabe, text, 10);		//Beschleunigung in text umwandeln

  if(ausgabe>max){			//Maximale Beschleunigung
    max = ausgabe;
    lcd_gotoxy(13,1,EN1);
    lcd_print(text,EN1);
  }

  if(ausgabe<min){			//Minimale Beschleunigung
    min = ausgabe;
    lcd_gotoxy(12,2,EN1);
    lcd_print(text,EN1);
  }

  lcd_gotoxy(2,1,EN1);
  lcd_print("    ",EN1);
  lcd_gotoxy(1,1,EN1);			//Aktuelle Beschleunigung ausgeben
  lcd_print(text,EN1);

}


int main(void)
{

  DDRB  = 0xFF;				// alles als Ausgang definieren fr LCD
  PORTB = 0x00;				// alles aus

  DDRD = 0x73;
  PORTD = 0x7F;

  delay_ms(30);

  lcd_init(EN1);			//LCD Initialisieren

  lcd_gotoxy(7,1,EN1);
  lcd_print("Max:", EN1);

  lcd_gotoxy(7,2,EN1);
  lcd_print("Min:", EN1);

  TIMSK = (1<<OCIE1A);			//16 bit timer Compare match aktiv
  TCCR1B = (0<<CS10 | 1<<CS11 | 0<<CS12 | 1<<CTC1);  //CLK/8 | Compare match

  OCR1AH = 0x00;  //00000000
  OCR1AL = 0x0A;  //00001010  Compare-Wert

  MCUCR = (1<<ISC11 | 1<<ISC00);	//Int1 rising edge
  GIMSK = (1<<INT1);			//Int1 aktiv

  sei();				//Globale Interrupts aktiv

  delay_ms(30);


  for (;;){	//Endlosschleife
    show_g();
    delay_ms(30);
  }
}

//############################
//#      LCD Functionen      #
//############################

//Daten bernehmen
void lcd_enable(char disp)
{
  sbi(PORTB, disp);
  delay_ms(1);
  cbi(PORTB, disp);
  delay_ms(1);
  PORTB = 0x00;
}

//Befehl senden
void lcd_command(char command, char disp)
{
  PORTB = command & 0xF0;
  lcd_enable(disp);
  PORTB = (command & 0x0F)<<4;
  lcd_enable(disp);
}


//Zeichen senden
void lcd_data(char data, char disp)
{
  PORTB = (data & 0xF0)+1;
  lcd_enable(disp);
  PORTB = ((data & 0x0F)<<4)+1;
  lcd_enable(disp);
  cbi(PORTB, RS);
}

//Position setzen
void lcd_gotoxy(char x, char y, char disp)
{
  lcd_command(pos[y-1][x-1]+0x80,disp);
}

void lcd_init(char disp){

  PORTB = 0x30;			// Initialisierung
  lcd_enable(disp);		// 3 mal 0x30
  delay_ms(5);
  lcd_enable(disp);
  delay_ms(1);
  lcd_enable(disp);
  delay_ms(1);

  PORTB = 0x20;			// 4 bit modus aktivieren
  lcd_enable(disp);

  lcd_command(0x28 ,disp);	// 4 zeilen; 4bit; 5x7Pixel (system set)

  lcd_command(0x01 ,disp);	// Clear

  lcd_command(0x02 ,disp);	// Cursor home

  lcd_command(0x06 ,disp);	// vorwrts inkrementieren; cursor nach rechts schreiben verschieben

  lcd_command(0x0C ,disp);	// Blinkender Unterstrich Cursor; Display an 0x0F

}

//Text senden
void lcd_print(char *text, char disp)
{
  while(*text != 0){
        lcd_data(*text++, disp);
  }
}
